/* Sistemas Integrados TEKNISA
  Gerao CFC (RNF) em Tabelas (Para outros pases)

Nome: 5014_SDXSTPO_GERA_CFC_EXTERIOR.SQL

@C:\web_509\sql\end_conv\oracle\clientes\sdx\5014_SDXSTPO_GERA_CFC_EXTERIOR.SQL

Descrio:
Rotina para gerao da CFC em tabelas.

P_CDFILIAL      - Cdigo da filial a ser gerado a CFC.
P_DATA          - Data de processamento para gerao da CFC.
P_ARQEXP        - Tipo de arquivo de exportao (SDX09310 / SDX09311 / SDX09312)
P_IDNMARQUIVO   - Identificador do nome do arquivo (CFC) - Parmetro no utilizado

PARA DEBUGAR
raise_application_error(-20501, '' , True);

Criao:
05/06/2009 - Fabrcio Marcelo da Silva
*/

CREATE OR REPLACE PROCEDURE EXPORTA_CFC
   (P_CDFILIAL      IN FILIAL.CDFILIAL%TYPE,
    P_DATA          IN DATE,
    P_ARQEXP        IN VARCHAR2,
    P_IDNMARQUIVO   IN PARAMFILIAL.IDINVROTDIA%TYPE  /*  Parmetro no utilizado  */
   ) IS

   V_CDTIPOOPER      VARCHAR2(10);    -- Tipos de operaes
   V_INSEQDOCU       INTEGER;         -- Cdigo sequncial gerado para cada novo documento (nota fiscal) exportado na tabela
   V_INSEQITEM       INTEGER;         -- Sequncial por documento (linhas do documento a ser exportado)

   V_NUMEROCFC       VARCHAR2(20);    -- Armazena nmero do CFC (RNF)
   V_QTRIMP          VARCHAR2(02);    -- Se possui outros impostos

   V_REFERE          VARCHAR2(32);
   V_CUEAUX          VARCHAR2(10);
BEGIN
   V_NUMEROCFC  := NULL;
   V_CDTIPOOPER := NULL;

   /*  Retorna o tipo de operao conforme exportao  */
   IF SUBSTR(P_ARQEXP,1,9) = 'SDX09310' THEN
      V_CDTIPOOPER := '_01_03_';
   ELSIF SUBSTR(P_ARQEXP,1,9) = 'SDX09311' THEN
      V_CDTIPOOPER := '_14_';
   ELSIF SUBSTR(P_ARQEXP,1,9) = 'SDX09312' THEN
      V_CDTIPOOPER := '_11_';
   END IF;

   /*  Retorna o nmero do CFC  */
   BEGIN
      SELECT NVL(SUBSTR(NRSEQUENCIAL, 11, 10),SUBSTR(NRSEQUENCIAL, 1, 10))
        INTO V_NUMEROCFC
        FROM NOVOCODIGO
       WHERE CDCONTADOR = 'RENOTAFISCAL'||P_CDFILIAL;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_NUMEROCFC := NULL;
   END;

   /*  Corpo da nota fiscal  */
   FOR C_NOTAFISCAL IN
      (SELECT N.CDFILIAL, N.NRLANCTONF, P.CDEMPINTEGRACAO, N.CDSERIEENTR, N.CDTIPOOPER,
              TO_CHAR(N.DTENTRSAID, 'DDMMYYYY') AS DTENTRSAID, TO_CHAR(N.DTEMISSAO, 'DDMMYYYY') AS DTEMISSAO,
              N.NRNOTAFISC, PA.CDEXFILIAL, PI.DSPROCEINVFMES, S.CDSERIEENTRNFE,
              SUBSTR(replace(replace(replace(N.NRINSJURNF, '-', ''),'.',''),'/',''),1,10) AS NRINSJURNF,
              N.VRNOTAFISC, DSCDFISOPENF, E.CDPAIS, N.CDFORNECED
         FROM NOTAFISCAL N, PARAMEMPRESA P, PARAMFILIAL PA, PARAMINTEGRA PI, SERIEENTR S, ENDEFILI E
        WHERE N.CDFILIRECE = P_CDFILIAL
          AND E.CDFILIAL   = N.CDFILIRECE
          AND E.IDTPENDEFILI = 'P'
          AND N.IDENTRSAIDOP = 'E'
          AND N.DTENTRSAID = P_DATA
          AND N.IDSTATUS <> 'C'
          AND NVL(N.IDSISTEMA, 'DFE') = 'DFE'
          AND INSTR(V_CDTIPOOPER, N.CDTIPOOPER) <> 0
          AND N.DTEXPRNF IS NULL
          AND P.CDEMPRESA = N.CDEMPRESA
          AND PA.CDFILIAL = P_CDFILIAL
          AND PI.CDPARAMINT = '1'
          AND S.CDSERIEENTR = N.CDSERIEENTR)
   LOOP
      V_INSEQDOCU := 0;
      V_INSEQITEM := 1;

      /*  Retorna maior sequncial  */
      SELECT NVL(MAX(CFC_CODIGO),0) + 1
        INTO V_INSEQDOCU
        FROM USR_CFC;

      /*  Se j existir a nota fiscal e caso a mesma no tenha sido exportada
          ser excluda da tabela USR_CFC  */
      DELETE FROM USR_CFC
       WHERE TEK_CDFILIAL = C_NOTAFISCAL.CDFILIAL
         AND TEK_NRLANCTONF = C_NOTAFISCAL.NRLANCTONF
         AND CFC_DTIMP IS NULL;

      /*Retorna a inscrio estadual do fornecedor da nota*/
      BEGIN
         SELECT SUBSTR(NRINSESTFORN,1,10)
           INTO V_CUEAUX
           FROM FORNECEDOR
          WHERE CDFORNECED = C_NOTAFISCAL.CDFORNECED;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_CUEAUX := NULL;
      END;
      /*Verifica se o pas da filial de recebimento  a Venezuela*/
      IF C_NOTAFISCAL.CDPAIS = '0058' THEN
         V_REFERE := REPLACE(C_NOTAFISCAL.DSCDFISOPENF,'&','|');
      ELSE
         V_REFERE := C_NOTAFISCAL.NRNOTAFISC;
      END IF;
      /*  Insere Corpo das Notas Fiscais - Linha Cabealho  */
      INSERT INTO USR_CFC
         (CFC_CODIGO, CFC_NUMLIN, CFC_NUEDOC, CFC_SOCIED, CFC_CLADOC,
          CFC_FECCON, CFC_FECDOC, CFC_REFERE, CFC_TEXCAB, CFC_MONDOC,
          CFC_CLACON, CFC_CUEAUX, CFC_MTODOC, CFC_ASIGNA, CFC_GLOSA,
          CFC_CCOSTO, CFC_CODIMP, CFC_CTAIMP, CFC_MONIMP, CFC_IMPREC,
          CFC_OTRIMP, CFC_DTIMP,  TEK_CDFILIAL, TEK_NRLANCTONF)
      VALUES
         (V_INSEQDOCU, V_INSEQITEM, 'X', SUBSTR(C_NOTAFISCAL.CDEMPINTEGRACAO,1,4), SUBSTR(C_NOTAFISCAL.CDSERIEENTR,1,2),
          C_NOTAFISCAL.DTENTRSAID, C_NOTAFISCAL.DTEMISSAO, V_REFERE, SUBSTR(C_NOTAFISCAL.CDEXFILIAL||'-'||V_NUMEROCFC,1,25), SUBSTR(C_NOTAFISCAL.DSPROCEINVFMES,1,5),
          SUBSTR(C_NOTAFISCAL.CDSERIEENTRNFE,1,2), V_CUEAUX, SUBSTR(replace(TO_CHAR(ROUND(C_NOTAFISCAL.VRNOTAFISC,2)),',','.'),1,11), SUBSTR(C_NOTAFISCAL.CDEXFILIAL,1,18), C_NOTAFISCAL.NRNOTAFISC,
          SUBSTR(C_NOTAFISCAL.CDEXFILIAL,1,10), NULL, NULL, NULL, NULL,
          NULL, NULL, C_NOTAFISCAL.CDFILIAL, C_NOTAFISCAL.NRLANCTONF);

      /*  Itens nota fiscal  */
      FOR C_ITEMNF IN
         (SELECT DECODE(I.CDOFICIMPO, 'C1', C_NOTAFISCAL.NRINSJURNF, '') AS ASIGNA,
                 DECODE(I.CDOFICIMPO, 'C1', P.CDCONTREDU, '') AS CDCONTREDU, DECODE(I.CDOFICIMPO, 'C1', P.NMCONTCTBL,'') AS NMCONTCTBL,
                 DECODE(I.CDOFICIMPO, 'C1', SUM(P.VRTOTAITEM),'') AS VRTOTAITEM,
                 I.CDOFICIMPO, DECODE(I.CDOFICIMPO, 'C0',P.CDCONTREDU, I.CDCONTIMP) AS CDCONTIMP, DECODE(C_NOTAFISCAL.CDTIPOOPER,'14','50','40') AS CLACON,
                 SUM(I.VRIMPOSTO) AS VRIMPOSTO, DECODE(I.CDOFICIMPO, 'C1', 'NO', 'SI') AS IMPREC,
                 NF.QTIMP
            FROM (SELECT NF.CDFILIAL, NF.NRLANCTONF, COUNT(CDIMPOSTO) AS QTIMP
                    FROM NOTAFISCAL NF, IMPOSTONF I
                   WHERE NF.CDFILIAL = C_NOTAFISCAL.CDFILIAL
                     AND NF.NRLANCTONF = C_NOTAFISCAL.NRLANCTONF
                     AND NF.CDFILIAL = I.CDFILIAL (+)
                     AND NF.NRLANCTONF = I.NRLANCTONF (+)
                   GROUP BY NF.CDFILIAL, NF.NRLANCTONF
                 ) NF,
                 (SELECT IT.CDFILIAL, IT.NRLANCTONF, IT.NRSEQITPROD, C.CDCONTREDU, C.NMCONTCTBL, IT.VRTOTAITEM
                    FROM ITEMPRODUTO IT, PRODUTO P, PRODUTO PR, CONTCTBL C
                   WHERE CDFILIAL = C_NOTAFISCAL.CDFILIAL
                     AND NRLANCTONF = C_NOTAFISCAL.NRLANCTONF
                     AND (P.CDARVPROD = (SELECT MAX(CDARVPROD)
                                           FROM  PRODUTO C
                                          WHERE (INSTR(PR.CDARVPROD,RTRIM(C.CDARVPROD)) = 1)
                                            AND NOT (CDCTBLENTRPR IS NULL)) OR IT.CDPRODUTO = P.CDPRODUTO)
                     AND IT.CDPRODUTO = PR.CDPRODUTO
                     AND C.CDCONTCTBL = P.CDCTBLENTRPR) P,
                 (SELECT IT.NRSEQITPROD, IT.CDFILIAL, IT.NRLANCTONF, C.CDCONTREDU AS CDCONTIMP, I.CDOFICIMPO, IT.VRIMPOSTOIT AS VRIMPOSTO
                    FROM ITEMPROIMPOS IT, IMPOSTO I, CONTCTBL C
                   WHERE IT.CDFILIAL = C_NOTAFISCAL.CDFILIAL
                     AND IT.NRLANCTONF = C_NOTAFISCAL.NRLANCTONF
                     AND I.CDIMPOSTO = IT.CDIMPOSTO
                     AND I.IDPRODNOTA = 'P'
                     AND C.CDCONTCTBL = I.CDCCTBLATIV

                   UNION ALL

                  SELECT '0001' AS NRSEQITPROD, IT.CDFILIAL, IT.NRLANCTONF, C.CDCONTREDU AS CDCONTIMP, I.CDOFICIMPO, IT.VRIMPOSTONF AS VRIMPOSTO
                    FROM IMPOSTONF IT, IMPOSTO I, CONTCTBL C
                   WHERE IT.CDFILIAL = C_NOTAFISCAL.CDFILIAL
                     AND IT.NRLANCTONF = C_NOTAFISCAL.NRLANCTONF
                     AND I.IDPRODNOTA = 'C'
                     AND I.CDIMPOSTO = IT.CDIMPOSTO
                     AND C.CDCONTCTBL = I.CDCCTBLATIV) I
           WHERE P.CDFILIAL = NF.CDFILIAL
             AND P.NRLANCTONF = NF.NRLANCTONF
             AND P.CDFILIAL = I.CDFILIAL (+)
             AND P.NRLANCTONF = I.NRLANCTONF (+)
             AND P.NRSEQITPROD = I.NRSEQITPROD (+)
           GROUP BY DECODE(I.CDOFICIMPO, 'C1', C_NOTAFISCAL.NRINSJURNF, ''), DECODE(I.CDOFICIMPO, 'C1', P.CDCONTREDU, ''),
                    DECODE(I.CDOFICIMPO, 'C1', P.NMCONTCTBL,''), I.CDOFICIMPO, DECODE(I.CDOFICIMPO, 'C0',P.CDCONTREDU, I.CDCONTIMP),
                    DECODE(C_NOTAFISCAL.CDTIPOOPER,'14','50','40'), DECODE(I.CDOFICIMPO, 'C1', 'NO', 'SI'), NF.QTIMP
           ORDER BY CDCONTREDU, CDOFICIMPO)
      LOOP
         V_QTRIMP := NULL;

         V_INSEQITEM := V_INSEQITEM + 1;

         /*  Se possuir outros impostos alm do IVA (C1) ser apresentado o valor igual a "SI",
             caso contrrio ser apresentado o valor "NO". Ou seja, para os outros impostos
             diferentes de IVA ser apresentado o valor fixo "NO".  */
         IF C_ITEMNF.CDOFICIMPO <> 'C1' OR C_ITEMNF.CDOFICIMPO IS NULL THEN
            V_QTRIMP := 'NO';
         ELSE
            IF C_ITEMNF.QTIMP > 1 THEN
               V_QTRIMP := 'SI';
            ELSE
               V_QTRIMP := 'NO';
            END IF;
         END IF;

         /*  Insere itens das Notas Fiscais - Linha Item  */
         INSERT INTO USR_CFC
            (CFC_CODIGO, CFC_NUMLIN, CFC_NUEDOC, CFC_SOCIED, CFC_CLADOC,
             CFC_FECCON, CFC_FECDOC, CFC_REFERE, CFC_TEXCAB, CFC_MONDOC,
             CFC_CLACON, CFC_CUEAUX, CFC_MTODOC, CFC_ASIGNA, CFC_GLOSA,
             CFC_CCOSTO, CFC_CODIMP, CFC_CTAIMP, CFC_MONIMP, CFC_IMPREC,
             CFC_OTRIMP, CFC_DTIMP,  TEK_CDFILIAL, TEK_NRLANCTONF)
         VALUES
            (V_INSEQDOCU, V_INSEQITEM, NULL, SUBSTR(C_NOTAFISCAL.CDEMPINTEGRACAO,1,4), SUBSTR(C_NOTAFISCAL.CDSERIEENTR,1,2),
             C_NOTAFISCAL.DTENTRSAID, C_NOTAFISCAL.DTEMISSAO, V_REFERE, SUBSTR(C_NOTAFISCAL.CDEXFILIAL||'-'||V_NUMEROCFC,1,25), SUBSTR(C_NOTAFISCAL.DSPROCEINVFMES,1,5),
             SUBSTR(C_ITEMNF.CLACON,1,2), SUBSTR(C_ITEMNF.CDCONTREDU,1,10), SUBSTR(replace(TO_CHAR(ROUND(C_ITEMNF.VRTOTAITEM,2)),',','.'),1,11), SUBSTR(C_ITEMNF.ASIGNA,1,18), SUBSTR(C_ITEMNF.NMCONTCTBL,1,50),

             SUBSTR(C_NOTAFISCAL.CDEXFILIAL,1,10), SUBSTR(C_ITEMNF.CDOFICIMPO,1,4), SUBSTR(C_ITEMNF.CDCONTIMP,1,10), SUBSTR(replace(TO_CHAR(ROUND(C_ITEMNF.VRIMPOSTO,2)),',','.'),1,11), SUBSTR(C_ITEMNF.IMPREC,1,2),
             SUBSTR(V_QTRIMP,1,2), NULL, C_NOTAFISCAL.CDFILIAL, C_NOTAFISCAL.NRLANCTONF);
      END LOOP;

      /*  Inclui ou altera a data e o nmero da CFC para os documentos  */
      UPDATE NOTAFISCAL
         SET DTEXPRNF = P_DATA,
             NREXPRNF = V_NUMEROCFC
       WHERE CDFILIAL = C_NOTAFISCAL.CDFILIAL
         AND NRLANCTONF = C_NOTAFISCAL.NRLANCTONF;
   END LOOP;
   COMMIT;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('EXPORTA_CFC','5.09.000'); END;
/

